@@ -19,6 +19,7 @@ class AlbumsWindow : public QMainWindow
 public:
     explicit AlbumsWindow(QWidget *parent = 0, QString token = "0");
     ~AlbumsWindow();
+    void browseAlbums(QString userId);
 
 private:
     Ui::AlbumsWindow *ui;
@@ -27,6 +28,7 @@ private:
     QFacebookReply *albumCreationReply;
     QNetworkAccessManager *nam;
     QString accessToken;
+    QString m_userId;
 
 private slots:
     void onReplyReceived();
