@@ -4,12 +4,15 @@
 #include <QMainWindow>
 #include <QDebug>
 #include <qxmpp/QXmppClient.h>
+#include <qxmpp/QXmppMessage.h>
 #include <qxmpp/QXmppConfiguration.h>
 #include <qxmpp/QXmppRosterManager.h>
 #include <qxmpp/QXmppLogger.h>
+#include "qxmpp/QXmppUtils.h"
 #include <qxmpp/QXmppVCardManager.h>
 #include <QMessageBox>
 #include <QSettings>
+#include <QTimer>
 #include "chat/vCardCache.h"
 #include "chat/rosterItem.h"
 #include "chat/rosterItemModel.h"
@@ -17,6 +20,7 @@
 #include "chat/rosterItemSortFilterProxyModel.h"
 #include "chat/capabilitiesCache.h"
 #include "chat/chatlogindialog.h"
+#include "chat/chatconversationwindow.h"
 
 namespace Ui {
     class ChatWindow;
@@ -30,16 +34,16 @@ public:
     explicit ChatWindow(QWidget *parent = 0);
     ~ChatWindow();
 
-signals:
-    void loginCanceled();
-
 private:
     Ui::ChatWindow *ui;
     QXmppClient *m_xmppClient;
     vCardCache *m_vCardCache;
     rosterItemModel *m_rosterItemModel;
     capabilitiesCache m_capabilitiesCache;
+    QString m_username;
+    QMap<QString, ChatConversationWindow*> m_conversationWindows;
     rosterItemSortFilterProxyModel *m_rosterItemSortFilterModel;
+    ChatConversationWindow* getChatWindow(const QString &);
 
 private slots:
     void onPresenceReceived(QXmppPresence presence);
@@ -48,6 +52,9 @@ private slots:
     void rosterChanged(QString bareJid);
     void updateVCard(QString bareJid);
     void onLoginClicked(QString username, QString password, bool save);
+    void onMessageReceived(QXmppMessage message);
+    void showChatWindow(const QString &);
+    void goOffline();
 };
 
 #endif // CHATWINDOW_H
