@@ -5,13 +5,17 @@ CheckinWindow::CheckinWindow(QWidget *parent, QString token) :
     QMainWindow(parent),
     ui(new Ui::CheckinWindow),
     accessToken(token),
-    qfacebook(new QFacebook(token, this))
+    qfacebook(new QFacebook(token, this)),
+    m_mapsWidget(new MapsWidget(this)),
+    markerManager(0)
 {
     ui->setupUi(this);
     setAttribute(Qt::WA_DeleteOnClose);
 #ifdef Q_WS_MAEMO_5
     setAttribute(Qt::WA_Maemo5StackedWindow);
 #endif
+    ui->mapLayout->addWidget(m_mapsWidget);
+    initializeMapWidget();
     connect(ui->tagFriendsEdit, SIGNAL(clicked()), this, SLOT(onTagFriendsClicked()));
     connect(ui->checkinButton, SIGNAL(clicked()), this, SLOT(onCheckinButtonClicked()));
     //connect(ui->checkinButton, SIGNAL(clicked()), this, SLOT(checkIn()));
@@ -22,16 +26,62 @@ CheckinWindow::~CheckinWindow()
     delete ui;
 }
 
+void CheckinWindow::initializeMapWidget()
+{
+    // check we have a valid default provider
+
+    QList<QString> providers = QGeoServiceProvider::availableServiceProviders();
+    if (providers.size() < 1) {
+        QMessageBox::information(this, tr("Maps Demo"),
+                                 tr("No service providers are available"));
+        QCoreApplication::quit();
+        return;
+    }
+
+    foreach (QString provider, providers) {
+        serviceProvider = new QGeoServiceProvider(provider);
+        if (serviceProvider->mappingManager() &&
+                serviceProvider->searchManager() &&
+                serviceProvider->routingManager())
+            break;
+    }
+
+    if (serviceProvider->error() != QGeoServiceProvider::NoError) {
+        QMessageBox::information(this, tr("Maps Demo"),
+                                 tr("Error loading geoservice plugin"));
+        QCoreApplication::quit();
+        return;
+    }
+
+    if (!serviceProvider->mappingManager() ||
+            !serviceProvider->searchManager() ||
+            !serviceProvider->routingManager()) {
+        QMessageBox::information(this, tr("Maps Demo"),
+                                 tr("No geoservice found with mapping/search/routing"));
+        QCoreApplication::quit();
+        return;
+    }
+
+    // start initialising things (maps, searching, routing)
+
+    m_mapsWidget->initialize(serviceProvider->mappingManager());
+    m_mapsWidget->setStyleSheet("border-radius: 9px;");
+
+    if (markerManager)
+        delete markerManager;
+    markerManager = new MarkerManager(serviceProvider->searchManager());
+    m_mapsWidget->setMarkerManager(markerManager);
+}
+
 void CheckinWindow::setLocation(QString name, QString id, QString longitude, QString latitude)
 {
     this->setWindowTitle(tr("Check into %1").arg(name));
-    ui->nameLabel->setText(name);
+    //ui->nameLabel->setText(name);
+    m_locationName = name;
     m_placeId = id;
     m_longitude = longitude;
     m_latitude = latitude;
-    qDebug() << longitude;
-    qDebug() << latitude;
-    //qDebug() << QString("Location set to %1 (id: %2) with lon: %3 lat: %4").arg(name, id, longitude, latitude);
+    m_mapsWidget->setMyLocation(QGeoCoordinate(latitude.toDouble(), longitude.toDouble()), true);
 }
 
 void CheckinWindow::onTagFriendsClicked()
@@ -71,7 +121,6 @@ void CheckinWindow::onCheckinButtonClicked()
     place.setLatitude(m_latitude);
     QVariantMap variant = QJson::QObjectHelper::qobject2qvariant(&place);
     QJson::Serializer serializer;
-    qDebug() << serializer.serialize(variant);
 
     data.append("&coordinates=" + serializer.serialize(variant));
     if (!ui->tagFriendsEdit->text().isEmpty())
@@ -96,7 +145,7 @@ void CheckinWindow::onCheckInReplyReceived()
     }
 
 #ifdef Q_WS_MAEMO_5
-    QMaemo5InformationBox::information(this, tr("Successfully checked into %1").arg(ui->nameLabel->text()));
+    QMaemo5InformationBox::information(this, tr("Successfully checked into %1").arg(m_locationName));
 #endif
 }
 
