@@ -27,6 +27,8 @@ public:
                         FacebookPrivacySettings,
                         FacebookHelpCenter};
     void openPage(FacebookBrowser::FacebookPages page);
+    void openPage(QString url);
+    QWebView* webView();
 
 private:
     Ui::FacebookBrowser *ui;
