@@ -16,12 +16,19 @@
 
 #include "cookiejar.h"
 
-/* TODO: Add Android define */
-#define perms "user_notes,friends_notes,publish_checkins,user_checkins,friends_checkins,create_event,user_events,rsvp_event,friends_events,user_groups,friends_groups,manage_pages,user_photos,friends_photos,read_stream,offline_access,publish_stream,read_mailbox,friends_birthday,friends_relationships,user_relationships,user_relationship_details,friends_relationship_details,user_location,friends_location,user_hometown,friends_hometown,user_activities,friends_activities,friends_status"
-#if defined (Q_WS_MAEMO_5) || defined (Q_WA_S60)
-    #define authUrl "https://www.facebook.com/dialog/oauth?client_id=198401840201289&redirect_uri=https://www.facebook.com/connect/login_success.html&scope=%1&response_type=token&display=touch"
+#ifdef Q_WS_MAEMO_5
+#define appId "198401840201289"
+#elif Q_WS_MAEMO_6
+#define appId "155528487852398"
 #else
-    #define authUrl "https://www.facebook.com/dialog/oauth?client_id=198401840201289&redirect_uri=https://www.facebook.com/connect/login_success.html&scope=%1&response_type=token"
+#define appId "133764606703287"
+#endif
+
+/* TODO: Add Android define */
+#if defined (Q_WS_MAEMO_5) || defined (Q_WA_S60)
+    #define authUrl "https://www.facebook.com/dialog/oauth?client_id=%2&redirect_uri=https://www.facebook.com/connect/login_success.html&scope=%1&response_type=token&display=touch"
+#else
+    #define authUrl "https://www.facebook.com/dialog/oauth?client_id=%2&redirect_uri=https://www.facebook.com/connect/login_success.html&scope=%1&response_type=token"
 #endif
 
 namespace Ui {
@@ -46,12 +53,14 @@ private:
     Ui::LoginWindow *ui;
     bool disableCloseButton;
     bool isLoggingIn;
+    QStringList permissionsList();
 
 private slots:
     void onUrlChanged(QUrl url);
     void onLoggedIn();
     void onLoadStarted();
     void onInitalLoadFinished(bool);
+    void onLoadFinished(bool);
 };
 
 #endif // LOGINWINDOW_H
