@@ -29,6 +29,7 @@ MainWindow::MainWindow(QWidget *parent) :
     friendsWindow(0),
     albumsWindow(0),
     profileWindow(0),
+    messagesWindow(0),
     groupsWindow(0),
     loginWindow(0),
     eventsWindow(0),
@@ -38,10 +39,18 @@ MainWindow::MainWindow(QWidget *parent) :
     nam(0)
 {
     ui->setupUi(this);
-    if (!QSettings().value("main/experimental-chat", false).toBool())
-        ui->listWidget->item(9)->setHidden(true);
     defaultItemCount = ui->listWidget->count();
+    isQuitting = false;
 #ifdef Q_WS_MAEMO_5
+    if (!QDBusConnection::sessionBus().registerService(DBUS_SERVICE)) {
+        qWarning("%s", qPrintable(QDBusConnection::sessionBus().lastError().message()));
+    }
+
+    if (!QDBusConnection::sessionBus().registerObject(DBUS_PATH, this, QDBusConnection::ExportScriptableSlots)) {
+        qWarning("%s", qPrintable(QDBusConnection::sessionBus().lastError().message()));
+    }
+    ui->menuBar->addAction(tr("Quit"), this, SLOT(quit()));
+
     setAttribute(Qt::WA_Maemo5StackedWindow);
     setAttribute(Qt::WA_Maemo5AutoOrientation);
 #endif
@@ -67,6 +76,8 @@ MainWindow::MainWindow(QWidget *parent) :
 #endif
         connect(notificationsDialog, SIGNAL(gotNotificationsCount(int)), this, SLOT(onGotNotificationCount(int)));
         connect(this, SIGNAL(refresh()), notificationsDialog, SLOT(updateNotifications()));
+        connect(this, SIGNAL(refresh()), this, SLOT(getNotifications()));
+        this->getNotifications();
     }
     int interval = QSettings().value("main/refresh-interval", 0).toInt();
     if (interval != 0) {
@@ -85,9 +96,10 @@ MainWindow::~MainWindow()
     delete ui;
 }
 
-void MainWindow::onRefreshTimeout()
+void MainWindow::refreshAll()
 {
-    emit refresh();
+    if (QNetworkConfigurationManager(this).isOnline())
+        emit refresh();
 }
 
 void MainWindow::saveFavorites()
@@ -103,7 +115,7 @@ void MainWindow::saveFavorites()
         FavoriteItem item;
         item.text = ui->listWidget->item(i)->text();
         item.avatar = ui->listWidget->item(i)->icon();
-        item.userId = ui->listWidget->item(i)->data(Qt::UserRole).toString();
+        item.userId = ui->listWidget->item(i)->data(Qt::UserRole).toString().remove("profile://");
         favorites.append(item);
     }
 
@@ -132,7 +144,7 @@ void MainWindow::loadFavorites()
     foreach (FavoriteItem item, favorites) {
         QListWidgetItem *favItem = new QListWidgetItem(ui->listWidget);
         favItem->setText(item.text);
-        favItem->setData(Qt::UserRole, item.userId);
+        favItem->setData(Qt::UserRole, QString("profile://" + item.userId));
         favItem->setIcon(item.avatar);
         ui->listWidget->addItem(favItem);
     }
@@ -151,7 +163,13 @@ void MainWindow::connectSignals()
     connect(ui->listWidget, SIGNAL(itemActivated(QListWidgetItem*)), this, SLOT(onItemActivated(QListWidgetItem*)));
     connect(ui->listWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(onContextMenuRequested(QPoint)));
     connect(QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
-    connect(m_refreshTimer, SIGNAL(timeout()), this, SLOT(onRefreshTimeout()));
+    connect(m_refreshTimer, SIGNAL(timeout()), this, SLOT(refreshAll()));
+#ifdef Q_WS_MAEMO_5
+    QDBusConnection::sessionBus().connect("com.nokia.HildonDesktop.AppMgr",
+                                          "/com/nokia/HildonDesktop/AppMgr",
+                                          "com.nokia.HildonDesktop.AppMgr",
+                                          "LaunchApplication", this, SLOT(onHildonLaunch(const QDBusMessage &message)));
+#endif
 }
 
 void MainWindow::showSettings()
@@ -209,11 +227,12 @@ void MainWindow::onItemActivated(QListWidgetItem *item)
         newsFeedWindow->show();
 #endif
     } else if (item->text() == tr("Messages")) {
-        MessagesWindow *window = new MessagesWindow(this, this->accessToken);
+        if (!messagesWindow)
+            messagesWindow = new MessagesWindow(this, this->accessToken);
 #ifdef Q_WS_S60
-        window->showMaximized();
+        messagesWindow->showMaximized();
 #else
-        window->show();
+        messagesWindow->show();
 #endif
     } else if (item->text() == tr("Photos")) {
         if (!albumsWindow) {
@@ -301,10 +320,10 @@ void MainWindow::onItemActivated(QListWidgetItem *item)
 #else
         notesWindow->show();
 #endif
-    } else if (!item->data(Qt::UserRole).isNull()) {
+    } else if (item->data(Qt::UserRole).toString().startsWith("profile://")) {
         ProfileWindow *window = new ProfileWindow(this, this->accessToken);
         window->setWindowTitle(item->text());
-        window->browseProfile(item->data(Qt::UserRole).toString());
+        window->browseProfile(item->data(Qt::UserRole).toString().remove("profile://"));
 #ifdef Q_WS_S60
         window->showMaximized();
 #else
@@ -324,19 +343,40 @@ void MainWindow::orientationChanged()
     ui->listWidget->setFlow(ui->listWidget->flow());
 }
 
-void MainWindow::getRequestsCount()
+void MainWindow::getNotifications()
 {
-    QString url = QString("https://api.facebook.com/method/notifications.get&access_token=%1").arg(accessToken);
     if (!nam)
         nam = new QNetworkAccessManager(this);
+    QString url = QString("https://api.facebook.com/method/notifications.get&format=JSON&access_token=%1").arg(accessToken);
     requestsReply = nam->get(QNetworkRequest(url));
     if (requestsReply)
-        connect(requestsReply, SIGNAL(finished()), this, SLOT(onGotRequestCount()));
+        connect(requestsReply, SIGNAL(finished()), this, SLOT(onGotNotifications()));
 }
 
-void MainWindow::onGotRequestCount()
+void MainWindow::onGotNotifications()
 {
+    if (requestsReply->error() != QNetworkReply::NoError) {
+        qDebug() << "Failed to fetch requests/notifications!";
+        requestsReply->deleteLater();
+        requestsReply = 0;
+        return;
+    }
 
+    QByteArray reply = requestsReply->readAll();
+    QJson::Parser parser;
+    QVariant jsonData = parser.parse(reply);
+    int newMessagesCount = jsonData.toMap().value("messages").toMap().value("unread").toInt();
+    int pokes = jsonData.toMap().value("pokes").toMap().value("unread").toInt();
+    int friendRequests = jsonData.toMap().value("friend_requests_counts").toMap().value("unread").toInt();
+    int eventInvitesCount = jsonData.toMap().value("event_invites").toList().count();
+    qDebug() << QString("%1 unread messages").arg(newMessagesCount);
+    qDebug() << QString("%1 pokes").arg(pokes);
+    qDebug() << QString("%1 friend requests").arg(friendRequests);
+    qDebug() << QString("%1 event invitations").arg(eventInvitesCount);
+    // Since positions are hardcoded for now...
+    ui->listWidget->item(2)->setData(Qt::UserRole, friendRequests);
+    ui->listWidget->item(3)->setData(Qt::UserRole, newMessagesCount);
+    ui->listWidget->item(6)->setData(Qt::UserRole, eventInvitesCount);
 }
 
 void MainWindow::onGotNotificationCount(int count)
@@ -364,6 +404,10 @@ void MainWindow::createLoginWindow()
 void MainWindow::onLoggedIn(QString token)
 {
     this->accessToken = token;
+    this->getNotifications();
+    notificationsDialog = new NotificationsDialog(this, accessToken);
+    connect(ui->pushButton, SIGNAL(clicked()), notificationsDialog, SLOT(show()));
+    connect(notificationsDialog, SIGNAL(gotNotificationsCount(int)), this, SLOT(onGotNotificationCount(int)));
     this->getMyId();
     QSettings().setValue("main/token", token);
     loginWindow = 0;
@@ -408,9 +452,6 @@ void MainWindow::onGetMyIdReply()
     QSettings().setValue("main/myId", this->myId);
     disconnect(reply, SIGNAL(finished()), this, SLOT(onGetMyIdReply()));
     qfacebook->deleteLater();
-    notificationsDialog = new NotificationsDialog(this, accessToken);
-    connect(ui->pushButton, SIGNAL(clicked()), notificationsDialog, SLOT(show()));
-    connect(notificationsDialog, SIGNAL(gotNotificationsCount(int)), this, SLOT(onGotNotificationCount(int)));
 }
 
 void MainWindow::onAddToFavoritesClicked()
@@ -437,7 +478,6 @@ void MainWindow::onFriendSelected(QList<QListWidgetItem*> list)
     Friend *user = new Friend(ui->listWidget, item->data(Qt::UserRole).toString());
     connect(user, SIGNAL(avatarProcessed(QString,QImage)), this, SLOT(onAvatarProcessed(QString,QImage)));
     QString avatarUrl = QString("http://graph.facebook.com/%1/picture&type=large").arg(item->data(Qt::UserRole).toString());
-    qDebug() << avatarUrl;
     if (!nam)
         nam = new QNetworkAccessManager(this);
     QNetworkReply *reply =
@@ -445,7 +485,7 @@ void MainWindow::onFriendSelected(QList<QListWidgetItem*> list)
     connect(reply, SIGNAL(finished()), user, SLOT(avatarReceived()));
 
     favItem->setText(item->text());
-    favItem->setData(Qt::UserRole, item->data(Qt::UserRole));
+    favItem->setData(Qt::UserRole, QString("profile://" + item->data(Qt::UserRole).toString()));
 }
 
 void MainWindow::onAvatarProcessed(QString id, QImage image)
@@ -453,7 +493,7 @@ void MainWindow::onAvatarProcessed(QString id, QImage image)
     QPixmap expandedPixmap = QPixmap::fromImage(image);
     expandedPixmap = expandedPixmap.scaled(ui->listWidget->size(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
     for (int i = 0; i < ui->listWidget->count(); i++) {
-        if (ui->listWidget->item(i)->data(Qt::UserRole).toString() == id)
+        if (ui->listWidget->item(i)->data(Qt::UserRole).toString().endsWith(id))
             ui->listWidget->item(i)->setIcon(QIcon(expandedPixmap));
     }
 }
@@ -480,11 +520,27 @@ void MainWindow::deleteWindows()
         delete chatWindow;
         chatWindow = 0;
     }
+    if (groupsWindow) {
+        delete groupsWindow;
+        groupsWindow = 0;
+    }
+    if (notesWindow) {
+        delete notesWindow;
+        notesWindow = 0;
+    }
+    if (messagesWindow) {
+        delete messagesWindow;
+        messagesWindow = 0;
+    }
     if (notificationsDialog) {
         delete notificationsDialog;
         notificationsDialog = 0;
         ui->pushButton->setText(tr("Notifications"));
     }
+
+    ui->listWidget->item(2)->setData(Qt::UserRole, 0);
+    ui->listWidget->item(3)->setData(Qt::UserRole, 0);
+    ui->listWidget->item(6)->setData(Qt::UserRole, 0);
 }
 
 #ifdef Q_WS_MAEMO_5
@@ -514,10 +570,25 @@ void MainWindow::takeScreenshot()
     XFlush (xev.xclient.display);
     XSync (xev.xclient.display, False);
 }
+
+void MainWindow::onHildonLaunch(const QDBusMessage &message)
+{
+    qDebug() << message.arguments();
+    this->show();
+    if (message.arguments()[0] == QApplication::applicationName())
+        this->top_application();
+}
 #endif
 
 void MainWindow::top_application()
 {
+    if (this->isHidden()) {
+#ifdef Q_WS_S60
+        this->showMaximized();
+#else
+        this->show();
+#endif
+    }
     this->activateWindow();
 }
 
@@ -541,3 +612,23 @@ void MainWindow::openHelpCenter()
     browser->openPage(FacebookBrowser::FacebookHelpCenter);
     browser->show();
 }
+
+void MainWindow::quit()
+{
+    isQuitting = true;
+    this->close();
+}
+
+void MainWindow::closeEvent(QCloseEvent *e)
+{
+#ifdef Q_WS_MAEMO_5
+    if (QSettings().value("main/hide-on-exit", false).toBool() && !isQuitting) {
+        e->ignore();
+        this->hide();
+    } else {
+        e->accept();
+    }
+#else
+    e->accept();
+#endif
+}
